package javax.swing.tree;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.Vector;
import java.io.ObjectOutputStream;
import java.util.EventListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import java.beans.ConstructorProperties;
import javax.swing.event.EventListenerList;
import java.io.Serializable;

public class DefaultTreeModel implements Serializable, TreeModel
{
    protected TreeNode root;
    protected EventListenerList listenerList;
    protected boolean asksAllowsChildren;
    
    @ConstructorProperties({ "root" })
    public DefaultTreeModel(TreeNode treeNode) {
        this(treeNode, false);
    }
    
    public DefaultTreeModel(TreeNode root, boolean asksAllowsChildren) {
        this.listenerList = new EventListenerList();
        this.root = root;
        this.asksAllowsChildren = asksAllowsChildren;
    }
    
    public void setAsksAllowsChildren(boolean asksAllowsChildren) {
        this.asksAllowsChildren = asksAllowsChildren;
    }
    
    public boolean asksAllowsChildren() {
        return this.asksAllowsChildren;
    }
    
    public void setRoot(TreeNode root) {
        TreeNode root2 = this.root;
        this.root = root;
        if (root == null && root2 != null) {
            this.fireTreeStructureChanged(this, null);
        }
        else {
            this.nodeStructureChanged(root);
        }
    }
    
    @Override
    public Object getRoot() {
        return this.root;
    }
    
    @Override
    public int getIndexOfChild(Object o, Object o2) {
        if (o == null || o2 == null) {
            return -1;
        }
        return ((TreeNode)o).getIndex((TreeNode)o2);
    }
    
    @Override
    public Object getChild(Object o, int n) {
        return ((TreeNode)o).getChildAt(n);
    }
    
    @Override
    public int getChildCount(Object o) {
        return ((TreeNode)o).getChildCount();
    }
    
    @Override
    public boolean isLeaf(Object o) {
        if (this.asksAllowsChildren) {
            return !((TreeNode)o).getAllowsChildren();
        }
        return ((TreeNode)o).isLeaf();
    }
    
    public void reload() {
        this.reload(this.root);
    }
    
    @Override
    public void valueForPathChanged(TreePath treePath, Object userObject) {
        MutableTreeNode mutableTreeNode = (MutableTreeNode)treePath.getLastPathComponent();
        mutableTreeNode.setUserObject(userObject);
        this.nodeChanged(mutableTreeNode);
    }
    
    public void insertNodeInto(MutableTreeNode mutableTreeNode, MutableTreeNode mutableTreeNode2, int n) {
        mutableTreeNode2.insert(mutableTreeNode, n);
        this.nodesWereInserted(mutableTreeNode2, new int[] { n });
    }
    
    public void removeNodeFromParent(MutableTreeNode mutableTreeNode) {
        MutableTreeNode mutableTreeNode2 = (MutableTreeNode)mutableTreeNode.getParent();
        if (mutableTreeNode2 == null) {
            throw new IllegalArgumentException("node does not have a parent.");
        }
        int[] array = { 0 };
        Object[] array2 = { null };
        mutableTreeNode2.remove(array[0] = mutableTreeNode2.getIndex(mutableTreeNode));
        array2[0] = mutableTreeNode;
        this.nodesWereRemoved(mutableTreeNode2, array, array2);
        return;
    }
    
    public void nodeChanged(TreeNode treeNode) {
        if (this.listenerList != null && treeNode != null) {
            TreeNode parent = treeNode.getParent();
            if (parent != null) {
                int index = parent.getIndex(treeNode);
                if (index != -1) {
                    this.nodesChanged(parent, new int[] { index });
                }
            }
            else if (treeNode == this.getRoot()) {
                this.nodesChanged(treeNode, (int[])null);
            }
        }
    }
    
    public void reload(TreeNode treeNode) {
        if (treeNode != null) {
            this.fireTreeStructureChanged(this, this.getPathToRoot(treeNode), null, (Object[])null);
        }
    }
    
    public void nodesWereInserted(TreeNode treeNode, int[] array) {
        if (this.listenerList != null && treeNode != null && array != null && array.length > 0) {
            int length = array.length;
            Object[] array2 = new Object[length];
            for (int i = 0; i < length; ++i) {
                array2[i] = treeNode.getChildAt(array[i]);
            }
            this.fireTreeNodesInserted(this, this.getPathToRoot(treeNode), array, array2);
        }
    }
    
    public void nodesWereRemoved(TreeNode treeNode, int[] array, Object[] array2) {
        if (treeNode != null && array != null) {
            this.fireTreeNodesRemoved(this, this.getPathToRoot(treeNode), array, array2);
        }
    }
    
    public void nodesChanged(TreeNode treeNode, int[] array) {
        if (treeNode != null) {
            if (array != null) {
                int length = array.length;
                if (length > 0) {
                    Object[] array2 = new Object[length];
                    for (int i = 0; i < length; ++i) {
                        array2[i] = treeNode.getChildAt(array[i]);
                    }
                    this.fireTreeNodesChanged(this, this.getPathToRoot(treeNode), array, array2);
                }
            }
            else if (treeNode == this.getRoot()) {
                this.fireTreeNodesChanged(this, this.getPathToRoot(treeNode), null, (Object[])null);
            }
        }
    }
    
    public void nodeStructureChanged(TreeNode treeNode) {
        if (treeNode != null) {
            this.fireTreeStructureChanged(this, this.getPathToRoot(treeNode), null, (Object[])null);
        }
    }
    
    public TreeNode[] getPathToRoot(TreeNode treeNode) {
        return this.getPathToRoot(treeNode, 0);
    }
    
    protected TreeNode[] getPathToRoot(TreeNode treeNode, int n) {
        TreeNode[] pathToRoot;
        if (treeNode == null) {
            if (n == 0) {
                return null;
            }
            pathToRoot = new TreeNode[n];
        }
        else {
            ++n;
            if (treeNode == this.root) {
                pathToRoot = new TreeNode[n];
            }
            else {
                pathToRoot = this.getPathToRoot(treeNode.getParent(), n);
            }
            pathToRoot[pathToRoot.length - n] = treeNode;
        }
        return pathToRoot;
    }
    
    @Override
    public void addTreeModelListener(TreeModelListener l) {
        this.listenerList.add(TreeModelListener.class, l);
    }
    
    @Override
    public void removeTreeModelListener(TreeModelListener l) {
        this.listenerList.remove(TreeModelListener.class, l);
    }
    
    public TreeModelListener[] getTreeModelListeners() {
        return (TreeModelListener[])this.listenerList.getListeners(TreeModelListener.class);
    }
    
    protected void fireTreeNodesChanged(Object source, Object[] path, int[] childIndices, Object[] children) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeModelListener.class) {
                if (o == null) {
                    o = new TreeModelEvent(source, path, childIndices, children);
                }
                ((TreeModelListener)listenerList[i + 1]).treeNodesChanged(o);
            }
        }
    }
    
    protected void fireTreeNodesInserted(Object source, Object[] path, int[] childIndices, Object[] children) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeModelListener.class) {
                if (o == null) {
                    o = new TreeModelEvent(source, path, childIndices, children);
                }
                ((TreeModelListener)listenerList[i + 1]).treeNodesInserted(o);
            }
        }
    }
    
    protected void fireTreeNodesRemoved(Object source, Object[] path, int[] childIndices, Object[] children) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeModelListener.class) {
                if (o == null) {
                    o = new TreeModelEvent(source, path, childIndices, children);
                }
                ((TreeModelListener)listenerList[i + 1]).treeNodesRemoved(o);
            }
        }
    }
    
    protected void fireTreeStructureChanged(Object source, Object[] path, int[] childIndices, Object[] children) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeModelListener.class) {
                if (o == null) {
                    o = new TreeModelEvent(source, path, childIndices, children);
                }
                ((TreeModelListener)listenerList[i + 1]).treeStructureChanged(o);
            }
        }
    }
    
    private void fireTreeStructureChanged(Object source, TreePath path) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeModelListener.class) {
                if (o == null) {
                    o = new TreeModelEvent(source, path);
                }
                ((TreeModelListener)listenerList[i + 1]).treeStructureChanged(o);
            }
        }
    }
    
    public <T extends EventListener> T[] getListeners(Class<T> t) {
        return this.listenerList.getListeners(t);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Vector obj = new Vector();
        objectOutputStream.defaultWriteObject();
        if (this.root != null && this.root instanceof Serializable) {
            obj.addElement("root");
            obj.addElement(this.root);
        }
        objectOutputStream.writeObject(obj);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        Vector<Object> vector = (Vector)objectInputStream.readObject();
        int index = 0;
        if (index < vector.size() && vector.elementAt(index).equals("root")) {
            this.root = (TreeNode)vector.elementAt(++index);
            ++index;
        }
    }
}
